package com.example.tfat.myapplication

import android.app.Activity
import android.app.ProgressDialog
import android.content.DialogInterface
import android.content.Intent
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.LinearLayoutManager
import android.os.Bundle
import android.graphics.drawable.Drawable
import android.support.v7.widget.RecyclerView
import android.view.*
import android.widget.PopupWindow
import com.example.tfat.myapplication.navigationbar.NavigationBarView
import com.example.tfat.myapplication.navigationbar.actor.ActorAdapter
import com.example.tfat.myapplication.navigationbar.scene.AddSceneAdapter
import com.example.tfat.myapplication.navigationbar.scene.AddSceneView
import com.example.tfat.myapplication.navigationbar.scene.ControlSceneView
import com.example.tfat.myapplication.navigationbar.scene.SceneAdapter
import com.bomostory.sceneeditmodule.navigationbar.actor.SelectActorView
import com.google.gson.Gson
import kotlinx.android.synthetic.main.activity_scene_edit.*
import com.bomostory.sceneeditmodule.basicdata.*
import com.bomostory.sceneeditmodule.navigationbar.dialogue.DialogueView
import com.bomostory.sceneeditmodule.view.EditActorView
import com.bomostory.sceneeditmodule.view.OnTouchBoMoSceneListener
import com.example.tfat.myapplication.navigationbar.actor.LayerManagementDialog
import kotlinx.android.synthetic.main.control_actor_dialog.view.*
import com.bomostory.sceneeditmodule.utils.FileUtils
import com.bomostory.sceneeditmodule.view.LayerView
import android.util.DisplayMetrics
import com.bomostory.sceneeditmodule.MovieEditActivity
import com.example.tfat.myapplication.navigationbar.EditSceneView
import com.example.tfat.myapplication.navigationbar.RecordFinishView
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.CopyOnWriteArrayList

class SceneEditActivity : AppCompatActivity(), ActorAdapter.OnActorDragListener, EditActorView.OnActorChangeListener,
        OnTouchBoMoSceneListener.OnSceneMoveListener, DialogInterface.OnDismissListener, FileUtils.OnSaveActorImage,
        LayerManagementDialog.OnLayerChange {

    private lateinit var project: Project
    private var currentSceneIndex = 0
    private var currentLayerIndex = 6
    private var startRecordTime = 0L
    private var swipeX = 0
    private var record = Record()
    private var resourceThemeBitmap = ArrayList<String>()
    private var resourceActorBitmap = ArrayList<ArrayList<String>>()
    private lateinit var sceneAdapter: SceneAdapter
    private var isRecord = false
    private var layerManagementDialog = LayerManagementDialog()
    private val monitorSize = DisplayMetrics()
    companion object {
        const val PHOTO_FROM_GALLERY = 1
        const val LAYER_MANAGEMENT = "layer_management"
        const val ACTOR_HEIGHT = 200
        const val ACTOR_WIDTH = 200
        const val ACTOR_INIT_POSITION_X = 200
        const val ACTOR_INIT_POSITION_Y = 200
        const val BACKGROUND_MOVE_RATE = 32
        val listeners = CopyOnWriteArrayList<Runnable>()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_scene_edit)
        window.setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                WindowManager.LayoutParams.FLAG_FULLSCREEN)
        actionBar?.hide()
        supportActionBar?.hide()
        project = Gson().fromJson<Project>(intent.getStringExtra("project"), Project::class.java)
        resourceThemeBitmap = project.sceneResource as ArrayList<String>
        resourceActorBitmap = project.actorResource as ArrayList<ArrayList<String>>
        windowManager.defaultDisplay.getMetrics(monitorSize)
        project.story?.let {
            it.scenes?.let{
                it[0].sceneWidth = monitorSize.widthPixels
                sceneEditView.scene = it[0]
            }
        }
        ic_scene_edit_activity_back.setOnClickListener {
            val pd = ProgressDialog(this).apply {
                setCancelable(false)
            }
            FileUtils.saveProject(project, 1920,1080)
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .doOnSubscribe { pd.show() }
                    .doFinally {
                        pd.dismiss()
                        this.onBackPressed()}
                    .subscribe {
                        listeners.forEach {it.run()}
                    }
        }
        initControlBarView()
        initNavigationBarView()
        initRecord()
    }

    override fun onResume() {
        super.onResume()
        window.decorView.apply {
            systemUiVisibility = (View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_LOW_PROFILE
                    or View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY)
        }
    }

    private fun initControlBarView() {
        controlBarView.setBtnSelected(6)
        switchToPreViewLayer()
        controlBarView.setonClickSwipeLayerBtn(View.OnClickListener {
            switchToPreViewLayer()
        })
        controlBarView.setonClickLayerSceneBtn(View.OnClickListener {
            controlBarView.setBtnSelected(5)
            layerManagementDialog = LayerManagementDialog()
            project.story?.let {
                it.scenes?.let{
                    layerManagementDialog.setData(it[currentSceneIndex])
                }
            }
            layerManagementDialog.setListener(this)
            layerManagementDialog.show(supportFragmentManager,LAYER_MANAGEMENT)
        })
        controlBarView.setonClickLayerBackgroundBtn(View.OnClickListener {
            switchLayer(4)
        })
        controlBarView.setonClickLayer4Btn(View.OnClickListener {
            switchLayer(3)
        })
        controlBarView.setonClickLayer3Btn(View.OnClickListener {
            switchLayer(2)
        })
        controlBarView.setonClickLayer2Btn(View.OnClickListener {
            switchLayer(1)
        })
        controlBarView.setonClickLayer1Btn(View.OnClickListener {
            switchLayer(0)
        })
    }

    //TFAT
    private fun initNavigationBarView() {
        project.story?.let {
            it.scenes?.let {
                if (it[currentSceneIndex].record == null) {
                    var navigationBarView = NavigationBarView(this)
                    navigationBar.addView(navigationBarView)
                    navigationBarView.setonClickSceneBtn(View.OnClickListener {
                        initControlSceneView()
                    })
                    navigationBarView.setonClickActorBtn(View.OnClickListener {
                        initActorRecyclerView()
                    })
                    navigationBarView.setonClickBrushBtn(View.OnClickListener {

                    })
                    navigationBarView.setonClickDialogueBtn(View.OnClickListener {
                        initDialogueView()
                    })
                    navigationBarView.setonClickImageBtn(View.OnClickListener {
                        pickFromGallery()
                    })
                } else {
                    navigationBar.removeAllViews()
                    var editSceneView = EditSceneView(this)
                    navigationBar.addView(editSceneView)
                    setControlSceneView()
                    editSceneView.setOnClickDoneListener(View.OnClickListener {
                        val pd = ProgressDialog(this).apply {
                                     setCancelable(false)
                                 }
                    FileUtils.saveProject(project, 1920,1080)
                             .subscribeOn(Schedulers.io())
                             .observeOn(AndroidSchedulers.mainThread())
                             .doOnSubscribe { pd.show() }
                             .doFinally { pd.dismiss() }
                             .subscribe {
                                 listeners.forEach {it.run()}
                             }
                    val intent = Intent(this, MovieEditActivity::class.java)
                    intent.putExtra(MovieEditActivity.PROJECT_KEY, Gson().toJson(project))
                    startActivity(intent)
                    })
                    editSceneView.setOnClickStartOverListener(View.OnClickListener {
                        project.story?.let {
                            it.scenes?.let{
                                it[currentSceneIndex].record = null
                                sceneEditView.scene = it[currentSceneIndex]
                                initNavigationBarView()
                            }
                        }
                    })
                    editSceneView.setOnClickReplaceListener(View.OnClickListener {
                        viewContainer.removeAllViews()
                        var addSceneView = AddSceneView(this)
                        viewContainer.addView(addSceneView)
                        val layoutManager = LinearLayoutManager(this)
                        layoutManager.orientation = LinearLayoutManager.HORIZONTAL
                        var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
                        for (i in 0 until resourceThemeBitmap.size) {
                            onSceneSelectedArrayList.add(replaceScene(i))
                        }
                        var addSceneAdapter = AddSceneAdapter(this, resourceThemeBitmap, onSceneSelectedArrayList)
                        var newSceneRecyclerView = addSceneView.findViewById<RecyclerView>(R.id.new_scene_recycler_view)
                        newSceneRecyclerView.layoutManager = layoutManager
                        newSceneRecyclerView.adapter = addSceneAdapter
                    })
                    editSceneView.setOnClickDeleteListener(View.OnClickListener {
                        project.story?.let {
                            it.scenes?.let{
                                if (it.size <= 1) { return@let }
                                it.removeAt(currentSceneIndex)
                                when(currentSceneIndex >= it.size){
                                    true -> {
                                        currentSceneIndex = 0
                                        sceneEditView.scene = it[0]
                                    }
                                    false -> sceneEditView.scene = it[currentSceneIndex]
                                }
                                initNavigationBarView()
                                initControlSceneView()
                            }
                        }
                    })
                }
            }
        }
    }

    private fun initDialogueView(){
        if (viewContainer.getChildAt(0) is DialogueView) {
            viewContainer.removeAllViews()
            return
        }
        var dialogueView = DialogueView(this)
        switchLayer(0)
        sceneEditView.findViewById<LayerView>(currentLayerIndex).setOnDragListener(dragListener())
        viewContainer.addView(dialogueView)
    }

    private fun initActorRecyclerView() {
        if (viewContainer.getChildAt(0) is SelectActorView) {
            viewContainer.removeAllViews()
            return
        }
        if (currentLayerIndex == 6){
            switchLayer(0)
            return
        }
        var selectActorView = SelectActorView(this)
        val layoutManager = LinearLayoutManager(this)
        layoutManager.orientation = LinearLayoutManager.HORIZONTAL
        var actorAdapter = ActorAdapter(this, resourceActorBitmap, sceneEditView.findViewById(currentLayerIndex), this)
        var actorRecyclerView = selectActorView.findViewById<RecyclerView>(R.id.select_actor_recycler_view)
        actorRecyclerView.layoutManager = layoutManager
        actorRecyclerView.adapter = actorAdapter
        viewContainer.addView(selectActorView)
    }

    private fun initControlSceneView() {
        if (viewContainer.getChildAt(0) is ControlSceneView) {
            viewContainer.removeAllViews()
            return
        }
        setControlSceneView()
    }

    private fun setControlSceneView(){
        var projectBitmaps = ArrayList<Drawable>()
        var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
        project.story?.let {
            it.scenes?.let{
                for (i in 0 until it.size) {
                    onSceneSelectedArrayList.add(switchScene(i))
                    projectBitmaps.add(Drawable.createFromPath(it[i].backgroundPath))
                }
            }
        }
        sceneAdapter = SceneAdapter(this, projectBitmaps, onSceneSelectedArrayList, currentSceneIndex)
        var controlSceneView = ControlSceneView(this)
        controlSceneView.setonClickAddBtn(View.OnClickListener {
            viewContainer.removeAllViews()
            var addSceneView = AddSceneView(this)
            viewContainer.addView(addSceneView)
            val layoutManager = LinearLayoutManager(this)
            layoutManager.orientation = LinearLayoutManager.HORIZONTAL
            var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
            for (i in 0 until resourceThemeBitmap.size) {
                onSceneSelectedArrayList.add(addScene(i))
            }
            var addSceneAdapter = AddSceneAdapter(this, resourceThemeBitmap, onSceneSelectedArrayList)
            var newSceneRecyclerView = addSceneView.findViewById<RecyclerView>(R.id.new_scene_recycler_view)
            newSceneRecyclerView.layoutManager = layoutManager
            newSceneRecyclerView.adapter = addSceneAdapter
        })
        controlSceneView.setonClickDuplicateBtn(View.OnClickListener {
            project.story?.let {
                it.scenes?.let{
                    it.add(currentSceneIndex + 1 ,it[currentSceneIndex])
                }
            }
            var sceneAdapter = controlSceneView.findViewById<RecyclerView>(R.id.scene_recycler_view).adapter as SceneAdapter
            var projectBitmaps = ArrayList<Drawable>()
            var onSceneSelectedArrayList = ArrayList<View.OnClickListener>()
            project.story?.let {
                it.scenes?.let{
                    for (i in 0 until it.size) {
                        onSceneSelectedArrayList.add(switchScene(i))
                        projectBitmaps.add(Drawable.createFromPath(it[i].backgroundPath))
                    }
                }
            }
            sceneAdapter.update(projectBitmaps, onSceneSelectedArrayList)
        })
        val layoutManager = LinearLayoutManager(this)
        layoutManager.orientation = LinearLayoutManager.HORIZONTAL
        var sceneRecyclerView = controlSceneView.findViewById<RecyclerView>(R.id.scene_recycler_view)
        sceneRecyclerView.layoutManager = layoutManager
        sceneRecyclerView.adapter = sceneAdapter
        viewContainer.removeAllViews()
        viewContainer.addView(controlSceneView)
    }
    private fun initRecord(){
        startRecord.setOnClickListener(View.OnClickListener {
            if (!isRecord) {
                startRecord.setImageDrawable(resources.getDrawable(R.drawable.ic_btn_record_stop))
                startRecordTime = System.currentTimeMillis()
                isRecord = true
                switchToPreViewLayer()
                navigationBar.visibility = View.INVISIBLE
            } else {
                isRecord = false
                record.period = System.currentTimeMillis() - startRecordTime
                project.story?.let {
                    it.scenes?.let{
                        it[currentSceneIndex].record = record
                    }
                }
                startRecord.setImageDrawable(resources.getDrawable(R.drawable.ic_btn_record))
                navigationBar.visibility = View.VISIBLE
                initControlSceneView()
                initRecordFinishView()
            }
        })
    }


    private fun initRecordFinishView(){
        navigationBar.removeAllViews()
        var recordFinishView = RecordFinishView(this)
        recordFinishView.setOnNextSceneClickListener(View.OnClickListener {
            currentSceneIndex++
            project.story?.let {
                it.scenes?.let{
                    when(currentSceneIndex < it.size){
                        true -> {
                            switchToPreViewLayer()
                            sceneEditView.scene = it[currentSceneIndex]
                        }
                        false -> {
                            currentSceneIndex = 0
                            sceneEditView.scene = it[currentSceneIndex]
                        }
                    }
                }
            }
            initControlBarView()
            initNavigationBarView()
        })
        recordFinishView.setOnFinishClickListener(View.OnClickListener {
            initNavigationBarView()
        })
        navigationBar.addView(recordFinishView)
    }
    private fun switchScene(position: Int): View.OnClickListener {
        return View.OnClickListener {
            currentSceneIndex = position
            switchToPreViewLayer()
            project.story?.let {
                it.scenes?.let{
                    sceneEditView.scene = it[position]
                }
            }
            initControlBarView()
            initNavigationBarView()
        }
    }

    private fun switchLayer(layerIndex: Int){
        viewContainer.removeAllViews()
        currentLayerIndex = layerIndex
        controlBarView.setBtnSelected(4 - layerIndex)
        project.story?.let {
            it.scenes?.let {
                for (layer in it[currentSceneIndex].layers) {
                    when(it[currentSceneIndex].layers.indexOf(layer)) {
                        currentLayerIndex ->
                            for (actor in it[currentSceneIndex].layers[currentLayerIndex].actors) {
                                actor.isMoveable = true
                            }
                        else ->
                            for (actor in it[currentSceneIndex].layers[it[currentSceneIndex].layers.indexOf(layer)].actors) {
                                actor.isMoveable = false
                            }
                    }
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        sceneEditView.setOnTouchListener(null)
        sceneEditView.x = 0
        setActorDropLayer()
        sceneEditView.setLayerVisible(layerIndex)
    }

    private fun switchToPreViewLayer(){
        viewContainer.removeAllViews()
        currentLayerIndex = 6
        controlBarView.setBtnSelected(currentLayerIndex)
        project.story?.let {
            it.scenes?.let {
                for (layer in it[currentSceneIndex].layers) {
                    for (actor in layer.actors) {
                        actor.isMoveable = false
                        actor.isSelect = false
                    }
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        val boMoSceneListener = OnTouchBoMoSceneListener()
        boMoSceneListener.setCallBack(this)
        sceneEditView.setOnTouchListener(boMoSceneListener)
        sceneEditView.x = 0
        if (viewContainer.getChildAt(0) is SelectActorView) {
            viewContainer.removeAllViews()
        }
    }


    private fun setActorDropLayer() {
        if (viewContainer.getChildAt(0) is SelectActorView) {
            var selectActorView = SelectActorView(this)
            val layoutManager = LinearLayoutManager(this)
            layoutManager.orientation = LinearLayoutManager.HORIZONTAL
            var actorAdapter = ActorAdapter(this, resourceActorBitmap, sceneEditView.findViewById(currentLayerIndex), this)
            var actorRecyclerView = selectActorView.findViewById<RecyclerView>(R.id.select_actor_recycler_view)
            actorRecyclerView.layoutManager = layoutManager
            actorRecyclerView.adapter = actorAdapter
            viewContainer.addView(selectActorView)
        }
    }

    private fun addScene(position: Int): View.OnClickListener {
        return View.OnClickListener {
            var scene = Scene()
            for (i in 0 until 5){
                val layer = Layer()
                layer.id = i
                scene.layers.add(layer)
            }
            scene.backgroundPath = resourceThemeBitmap[position]
            project.story?.let {
                it.scenes?.let{
                    scene.sceneWidth = monitorSize.widthPixels
                    it.add(scene)
                }
            }
            initControlSceneView()
            initNavigationBarView()
        }
    }

    private fun replaceScene(position: Int): View.OnClickListener {
        return View.OnClickListener {
            project.story?.let {
                it.scenes?.let{
                    it[currentSceneIndex].backgroundPath = resourceThemeBitmap[position]
                    sceneEditView.scene = it[currentSceneIndex]
                }
            }
            initControlSceneView()
            initNavigationBarView()
        }
    }
    private fun setActorPositionZ(actors: ArrayList<Actor>){
        for (actor in actors) {
            actor.positionZ = actors.indexOf(actor)
        }
    }

    private fun pickFromGallery() {
        val intent = Intent()
        intent.type = "image/*"
        intent.action = Intent.ACTION_GET_CONTENT
        startActivityForResult(intent, PHOTO_FROM_GALLERY)
    }

    private fun addActor(positionX: Int, positionY: Int, resourcePath: String) {
        project.story?.let {
            it.scenes?.let{
                val actorData = Actor()
                actorData.positionX = positionX
                actorData.positionY = positionY
                actorData.positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                actorData.sideLength = ACTOR_WIDTH
                actorData.sideHeight = ACTOR_HEIGHT
                actorData.resourcePath = resourcePath
                actorData.parentLayerIndex = currentLayerIndex
                actorData.isMoveable = true
                it[currentSceneIndex].layers[currentLayerIndex].actors.add(actorData)
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        setActorDropLayer()
    }

    private fun addDialogue(positionX: Int, positionY: Int, resource: Int) {
        project.story?.let {
            it.scenes?.let{
                val dialog = Dialogue()
                dialog.resource = resource
                dialog.positionX = positionX
                dialog.positionY = positionY
                dialog.positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                dialog.sideLength = ACTOR_WIDTH
                dialog.sideHeight = ACTOR_HEIGHT
                dialog.parentLayerIndex = currentLayerIndex
                dialog.isMoveable = true
                it[currentSceneIndex].layers[currentLayerIndex].actors.add(dialog)
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        sceneEditView.findViewById<LayerView>(currentLayerIndex).setOnDragListener(dragListener())
    }
    override fun onActorDropFinish(positionX: Int, positionY: Int, resourcePath: String) {
        addActor(positionX, positionY, resourcePath)
    }

    override fun onActorChange(actor: Actor) {
        project.story?.let {
            it.scenes?.let {
                it[currentSceneIndex].layers[currentLayerIndex].actors[actor.positionZ] = actor
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
    }

    override fun onActorSelected(actor: Actor) {
        project.story?.let {
            it.scenes?.let {
                for(i in 0 until it[currentSceneIndex].layers[currentLayerIndex].actors.size){
                    if(actor.positionZ == i) {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect =
                                !it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect
                    } else {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect = false
                    }
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
    }

    override fun onActorLongClick(actor: Actor) {
        project.story?.let {
            it.scenes?.let {
                for(i in 0 until it[currentSceneIndex].layers[currentLayerIndex].actors.size){
                    if(actor.positionZ == i) {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect =
                                !it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect
                    } else {
                        it[currentSceneIndex].layers[currentLayerIndex].actors[i].isSelect = false
                    }
                }
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        var popupWindow = PopupWindow(this)
        val popupView = LayoutInflater.from(this).inflate(R.layout.control_actor_dialog, null)
        popupView.duplicate.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    val actorData = Actor().apply {
                        positionX = actor.positionX + 100
                        positionY = actor.positionY + 100
                        positionZ = it[currentSceneIndex].layers[currentLayerIndex].actors.size
                        sideLength = actor.sideLength
                        sideHeight = actor.sideHeight
                        resourcePath = actor.resourcePath
                        parentLayerIndex = currentLayerIndex
                        isMoveable = true
                    }
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(actorData)
                     setActorPositionZ(it[currentSceneIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.bring_to_front.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.remove(actor)
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(actor)
                    setActorPositionZ(it[currentLayerIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.send_to_back.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.remove(actor)
                    it[currentSceneIndex].layers[currentLayerIndex].actors.add(0, actor)
                    setActorPositionZ(it[currentLayerIndex].layers[currentLayerIndex].actors)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupView.delete.setOnClickListener{
            project.story?.let {
                it.scenes?.let {
                    it[currentSceneIndex].layers[currentLayerIndex].actors.removeAt(actor.positionZ)
                    sceneEditView.scene = it[currentSceneIndex]
                    popupWindow.dismiss()
                }
            }
        }
        popupWindow.contentView = popupView
        popupWindow.width = ViewGroup.LayoutParams.WRAP_CONTENT
        popupWindow.height = ViewGroup.LayoutParams.WRAP_CONTENT
        popupWindow.isOutsideTouchable = true
        popupWindow.showAtLocation(sceneEditView, Gravity.START,actor.positionX,actor.positionY)
    }


    override fun onSceneMove(dX: Int) {
        if ( (BACKGROUND_MOVE_RATE * 35) >= (swipeX + dX) && (swipeX + dX) > (-(BACKGROUND_MOVE_RATE * 35))) {
            sceneEditView.x = swipeX + dX
            if (isRecord) {
                record?.tracks.let {
                    val track = Track()
                    track.positionX = swipeX + dX
                    track.time = System.currentTimeMillis() - startRecordTime
                    record.tracks.add(track)
                }
            }
        }
    }

    override fun onSceneReadyToMove() {
        swipeX = sceneEditView.x
    }

    override fun onSceneFinishMove() {
        swipeX = 0
    }

    override fun onDismiss(dialog: DialogInterface?) {
        project.story?.let {
            it.scenes?.let {
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
        window.decorView.apply {
            systemUiVisibility = (View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                    or View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                    or View.SYSTEM_UI_FLAG_FULLSCREEN
                    or View.SYSTEM_UI_FLAG_LOW_PROFILE
                    or View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY)
        }
        initControlBarView()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            PHOTO_FROM_GALLERY ->
                when (resultCode) {
                    Activity.RESULT_OK -> if (data != null) {
                        FileUtils.saveImageForActor(this, data.data, System.currentTimeMillis().toString())
                    }
                    Activity.RESULT_CANCELED -> {
                    }
                }
        }
    }

    override fun onSaveActorImage(filePath: String) {
        addActor(ACTOR_INIT_POSITION_X, ACTOR_INIT_POSITION_Y, filePath)
    }

    override fun onLayerChange(layerFrom: Int, layerTo: Int, position: Int){
        project.story?.let {
            it.scenes?.let {
                var actor = it[currentSceneIndex].layers[layerFrom - 1].actors[position]
                it[currentSceneIndex].layers[layerTo - 1].actors.add(actor)
                it[currentSceneIndex].layers[layerFrom - 1].actors.removeAt(position)
                layerManagementDialog.setData(it[currentSceneIndex])
                setActorPositionZ(it[currentSceneIndex].layers[layerFrom - 1].actors)
                setActorPositionZ(it[currentSceneIndex].layers[layerTo - 1].actors)
                sceneEditView.scene = it[currentSceneIndex]
            }
        }
    }

    private fun dragListener() : View.OnDragListener {
        return View.OnDragListener {v, event ->
            when (event.action) {
                DragEvent.ACTION_DRAG_STARTED -> {
                    return@OnDragListener true
                }
                DragEvent.ACTION_DROP -> {
                    addDialogue(event.x.toInt() - 100, event.y.toInt() - 100, event.clipData.getItemAt(0).text.toString().toInt())
                    return@OnDragListener true
                }
            }
            return@OnDragListener true
        }
    }
}
